@@ -9,14 +9,16 @@
 **    May you share freely, never taking more than you give.
 **
 *************************************************************************
-** This file contains a set of tests for the sqlite3 JNI bindings.
+** This file contains the ValueHolder utility class for the sqlite3
+** JNI bindings.
 */
 package org.sqlite.jni.capi;
 
 /**
    A helper class which simply holds a single value. Its primary use
    is for communicating values out of anonymous classes, as doing so
-   requires a "final" reference.
+   requires a "final" reference, as well as communicating aggregate
+   SQL function state across calls to such functions.
 */
 public class ValueHolder<T> {
   public T value;
